#!/usr/bin/env ruby

require "twitter"
require 'getoptlong'

# == Twitter Back: A backup and restore facility for Twitter friends
#
# Twitter Back allows you to backup and restore your list of twitter friends
#
# == Usage
# twitter_back [OPTION]
#	--help, -h:
#		show help
#	--backup x, -b x:
#		backup your friend list to the specified file
#	--restore x, -r x:
#		restore your friend list from the specified file
#	--status, -s:
#		report on the number of API hits you have remaining
#	-v: verbose
#
# Author:: Robin Wood (dninja@gmail.com)
# Copyright:: Copyright (c) Robin Wood 2009
# Licence:: GPL
#

# Backup the list for the specified screen name to the specified file
def backup(screen_name, password, filename, verbose)
	puts "Backing up friend list for "+screen_name

	if verbose
		puts '', "Friends", "=" * 50
	end

	begin
		output=File.open(filename, "w")
	rescue
		puts "Unable to open backup file"
		puts "Error message: "+$!
		return
	end

	begin
		begin
			Twitter::Base.new(screen_name, password).friends.each do |u|
				if verbose
					puts u.screen_name
				end
				output.puts(u.screen_name)
			end
		rescue Twitter::CantConnect => error
			puts "Unable to authenticate to Twitter, check your screen name and password"
			puts "Error message from Twitter: "+error
			return
		end

		if verbose
			puts
		end
	rescue
		puts "Unknown error occured: "+$!
		return
	end
end

# Restore from the specified file to the specified screen name
def restore(screen_name, password, filename, verbose)
	puts "Restoring friend list for "+screen_name

	begin
		input=File.open(filename, "r")
	rescue
		puts $!
		return
	end

	input.each { |line|
		line.chomp!
		if verbose
			puts line.inspect
		end

		begin
			Twitter::Base.new(screen_name, password).create_friendship(line)
		rescue Twitter::CantConnect => error
			puts "Unable to authenticate to Twitter, check your screen name and password"
			puts "Error message from Twitter: "+error
			return
		rescue
			puts "Unknown error occured: "+$!
			return
		end
	}
end

# Show how many API hits you have left and when they will be refreshed
def status(screen_name, password)
	results = Twitter::Base.new(screen_name, password).rate_limit_status()
	puts "API hits: "+results.remaining_hits.to_s+"/"+results.hourly_limit.to_s
	puts "Time of next reset: "+results.reset_time.to_s
end

opts = GetoptLong.new(
	[ '--help', '-h', GetoptLong::NO_ARGUMENT ],
	[ '--status', '-s', GetoptLong::NO_ARGUMENT ],
	[ '--backup', "-b" , GetoptLong::REQUIRED_ARGUMENT ],
	[ '--restore', "-r" , GetoptLong::REQUIRED_ARGUMENT ],
	[ "-v" , GetoptLong::NO_ARGUMENT ]
)

# Display the usage
def usage
	puts"twitter_back 1.0 Robin Wood (dninja@gmail.com) (www.digininja.org)

Usage: twitter_back [OPTION]
	--help, -h: show help
	--backup x, -b x: backup your friend list to the specified file
	--restore x, -r x: restore your friend list from the specified file
	--status, -s: report on the number of API hits you have remaining
	-v: verbose

"
	exit
end

verbose = false
filename=nil
command=nil

begin
	opts.each do |opt, arg|
		case opt
		when '--help'
			usage
		when "--backup"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			filename=arg
			command="backup"
		when "--restore"
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			if File.exists?(arg)
				filename=arg
			else
				puts "Can't find file to restore from"
				puts
				usage()
				exit
			end
			command="restore"
		when '-v'
			verbose=true
		when '--status'
			if command != nil
				puts "You can only specify one command at a time\n\n"
				usage()
			end
			command="status"
		end
	end
rescue
	usage
end

if command==nil
	usage
end

puts "Enter your twitter screen nname"
screen_name=gets
puts "Enter your password"
system "stty -echo"
password=gets()
system "stty echo"

# the lines come in with carriage return on the end
screen_name.chomp!
password.chomp!

if screen_name==""
	puts "Empty screen name entered"
	exit
end

if password==""
	puts "Blank password entered"
	exit
end

case command
when "backup"
	backup(screen_name, password, filename, verbose)
when "restore"
	restore(screen_name, password, filename, verbose)
when "status"
	status(screen_name, password)
when nil
	puts "No command specified"
	usage()
end

puts "Finished\n"
